/*
 * JBoss, Home of Professional Open Source
 *
 * Copyright 2013 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.picketlink.test.identity.federation.core.parser.saml;

import org.junit.Test;
import org.picketlink.common.util.DocumentUtil;
import org.picketlink.common.util.StaxUtil;
import org.picketlink.identity.federation.core.parsers.saml.SAMLParser;
import org.picketlink.identity.federation.core.saml.v2.util.XMLTimeUtil;
import org.picketlink.identity.federation.core.saml.v2.writers.SAMLResponseWriter;
import org.picketlink.identity.federation.core.util.JAXPValidationUtil;
import org.picketlink.identity.federation.saml.v2.protocol.ArtifactResponseType;
import org.picketlink.identity.federation.saml.v2.protocol.AuthnRequestType;
import org.picketlink.identity.federation.saml.v2.protocol.ResponseType;
import org.picketlink.identity.federation.saml.v2.protocol.StatusType;
import org.w3c.dom.Document;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * Unit test the parsing of {@link ArtifactResponseType}
 *
 * @author Anil.Saldhana@redhat.com
 * @since Jul 1, 2011
 */
public class SAMLArtifactResponseParserTestCase {

    @Test
    public void testSAMLArtifactResponseWithAuthnRequestParse() throws Exception {
        String file = "parser/saml2/saml2-artifact-response-authnrequest.xml";
        ClassLoader tcl = Thread.currentThread().getContextClassLoader();
        InputStream configStream = tcl.getResourceAsStream(file);

        JAXPValidationUtil.validate(configStream);
        configStream = tcl.getResourceAsStream(file);

        SAMLParser parser = new SAMLParser();
        ArtifactResponseType artifactResponse = (ArtifactResponseType) parser.parse(configStream);
        assertNotNull("ArtifactResponseType is not null", artifactResponse);

        assertEquals("ID_d84a49e5958803dedcff4c984c2b0d95", artifactResponse.getID());
        assertEquals(XMLTimeUtil.parse("2004-12-05T09:21:59Z"), artifactResponse.getIssueInstant());
        assertEquals("ID_cce4ee769ed970b501d680f697989d14", artifactResponse.getInResponseTo());
        assertTrue(artifactResponse.getAny() instanceof AuthnRequestType);

        StatusType status = artifactResponse.getStatus();
        assertNotNull(status);
        assertEquals("urn:oasis:names:tc:SAML:2.0:status:Success", status.getStatusCode().getValue().toString());

        // Try out writing
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        SAMLResponseWriter writer = new SAMLResponseWriter(StaxUtil.getXMLStreamWriter(baos));
        writer.write(artifactResponse);

        ByteArrayInputStream bis = new ByteArrayInputStream(baos.toByteArray());
        Document doc = DocumentUtil.getDocument(bis); // throws exceptions
        JAXPValidationUtil.validate(DocumentUtil.getNodeAsStream(doc));
    }

    @Test
    public void testSAMLArtifactResponseWithResponseParse() throws Exception {
        String file = "parser/saml2/saml2-artifact-response-response.xml";
        ClassLoader tcl = Thread.currentThread().getContextClassLoader();
        InputStream configStream = tcl.getResourceAsStream(file);

        JAXPValidationUtil.validate(configStream);
        configStream = tcl.getResourceAsStream(file);

        SAMLParser parser = new SAMLParser();
        ArtifactResponseType artifactResponse = (ArtifactResponseType) parser.parse(configStream);
        assertNotNull("ArtifactResponseType is not null", artifactResponse);

        assertEquals("ID_d84a49e5958803dedcff4c984c2b0d95", artifactResponse.getID());
        assertEquals(XMLTimeUtil.parse("2004-12-05T09:21:59Z"), artifactResponse.getIssueInstant());
        assertEquals("ID_cce4ee769ed970b501d680f697989d14", artifactResponse.getInResponseTo());
        assertTrue(artifactResponse.getAny() instanceof ResponseType);

        StatusType status = artifactResponse.getStatus();
        assertNotNull(status);
        assertEquals("urn:oasis:names:tc:SAML:2.0:status:Success", status.getStatusCode().getValue().toString());

        // Try out writing
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        SAMLResponseWriter writer = new SAMLResponseWriter(StaxUtil.getXMLStreamWriter(baos));
        writer.write(artifactResponse);

        ByteArrayInputStream bis = new ByteArrayInputStream(baos.toByteArray());
        Document doc = DocumentUtil.getDocument(bis); // throws exceptions
        JAXPValidationUtil.validate(DocumentUtil.getNodeAsStream(doc));
    }
}